==> tecfy_corev2_package-1.0.1.tar.gz: lib/views/merge/entity_merge.page.dart <==
part of tecfy_corev2_client_package;

class EntityMergePage extends StatefulWidget {
  /// entity for merge
  final Entity entity;

  /// returns true if the user has merged anything
  const EntityMergePage({Key? key, required this.entity}) : super(key: key);

  @override
  _EntityMergePageState createState() => _EntityMergePageState();
}

class _EntityMergePageState extends TecfyState<EntityMergePage>
    with SingleTickerProviderStateMixin {
  String? propertyId;
  List<Map<String, dynamic>>? _data;
  Map<String, dynamic>? _selectedData;
  Map<String, dynamic>? _selectedDataDetail;
  late TabController _tabController;
  Future<void> _onOpenDetailPage(Map<String, dynamic> data) async {
    _selectedData = data;
    if (_selectedData != null) {
      _selectedDataDetail = _selectedData!['records'][0];
      _tabController.animateTo(1);
    }
  }

  Future<void> _onSave() async {
    var data = _data?.where((e) => e['selected'] == true).map((e) {
      return {
        'active': e['default']['_id'],
        'inactive': e['records']
            .where((r) => r['isDefault'] != true)
            .map((r) => r['_id'])
            .toList()
      };
    }).toList();
    if (kDebugMode) print(data);
    if (data == null || data.length == 0) {
      AppNavigator.showMessage(
          context, 'No Data to mirge', MessageType.warning);
      return;
    }
    var result = await TecfyBasicApp.apiService
        .httpPost('entity/setMerge/${widget.entity.dbName}', data, true);
    if (kDebugMode) print(result);
    if (result.success) {
      AppNavigator.pop(context, true);
    } else {
      AppNavigator.showMessage(context, 'Error', MessageType.error,
          message: result.message);
    }
  }

  Future<void> _onLoadData() async {
    AppNavigator.showLoading(context);
    var result = await TecfyBasicApp.apiService.httpGetDynamic(
        'entity/getMerge/${widget.entity.dbName}?property=$propertyId');
    AppNavigator.pop(context);
    if (result.success && result.data != null) {
      var dd = result.data;
      //print(_data);
      // for (var d in dd) {
      //   d['value'] = ;
      // }
      _data = dd.map<Map<String, dynamic>>((e) {
        return {
          'id': e['key'],
          'selected': false,
          'default': e['value'][0],
          'display': e['value'][0][propertyId],
          'recordsCount': e['value'].length,
          'records': e['value']
              .map<Map<String, dynamic>>((e) => e as Map<String, dynamic>)
              .toList(),
        };
      }).toList();
      setState(() {});
    } else {
      AppNavigator.showMessage(context, 'Error', MessageType.error,
          message: result.message);
    }
  }

  @override
  initState() {
    _tabController = TabController(length: 2, vsync: this);
    _tabController.addListener(() {
      setState(() {});
    });
    super.initState();
  }

  @override
  dispose() {
    _tabController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return BasePage(
      context,
      '${trans('Merge')} ${widget.entity.name}',
      widget.entity.dbName,
      isMobile: LayoutService.isMobile(context),
      appBarActions: [IconButton(icon: Icon(Icons.save), onPressed: _onSave)],
      appBarBottom: LayoutService.isMobile(context)
          ? _getTabBar()
          : Align(
              alignment: LocalConfigService.isArabic
                  ? Alignment.centerRight
                  : Alignment.centerLeft,
              child: Container(
                  child: _getTabBar(),
                  constraints: BoxConstraints(maxWidth: 300))),
      child: Align(
          alignment: Alignment.topCenter,
          child: Container(
              constraints:
                  BoxConstraints(maxWidth: ThemeService.containerWidth),
              child: TabBarView(controller: _tabController, children: [
                _getEntityList(),
                _getEntityMerge(),
              ]))),
    );
  }

  Widget _getEntityList() {
    return Column(children: [
      Row(
        children: [
          Container(
              constraints: BoxConstraints(maxWidth: 250),
              child: DropdownComponenet(
                label: 'Merge Field',
                options: widget.entity.properties
                    .where(
                        (e) => e.propertyType == PropertyTypes.Text && e.isEdit)
                    .map((e) => e.toJson())
                    .toList(),
                idProperty: 'dbName',
                displayProperty: 'nameAr',
                onChanged: (v) => propertyId = v,
              )),
          IconButton(icon: Icon(Icons.refresh), onPressed: _onLoadData),
        ],
      ),
      Expanded(
          child: AppDataTable(
        onRowTap: _onOpenDetailPage,
        isSelectedField: 'selected',
        columns: [
          PropertyText(
              dbName: 'display',
              nameAr: 'البيان',
              nameEn: 'Name',
              readOnly: true),
          PropertyNumber(
              dbName: 'recordsCount',
              nameAr: 'عدد السجلات',
              nameEn: 'Count',
              readOnly: true),
        ],
        data: _data ?? [],
      ))
    ]);
  }

  TabBar _getTabBar() {
    return TabBar(
        controller: _tabController,
        labelColor: ThemeService.mainColor,
        tabs: [
          Tab(child: TecfyText(trans('Merge Data'))),
          Tab(child: TecfyText(trans('Details'))),
        ]);
  }

  Widget _getEntityMerge() {
    if (_selectedData == null)
      return Center(
          child: TecfyText('Please Select '
              'Item'));
    return Row(
      crossAxisAlignment: CrossAxisAlignment.start,
      mainAxisAlignment: MainAxisAlignment.start,
      children: [
        Expanded(
            child: AppDataTable(
          readOnly: true,
          onRowTap: (v) {
            _selectedDataDetail = v;
            setState(() {});
          },
          isSelectedField: 'isDefault',
          onSelectedChanged: (data, selected) {
            if (selected) {
              for (var d in _selectedData!['records']) {
                if (d != data) d['isDefault'] = false;
              }
              _selectedData!['default'] = data;
              _selectedData!['display'] = data[propertyId];
              _selectedData!['selected'] = true;
            } else {
              _selectedData!['default'] = null;
            }
          },
          columns: widget.entity.properties
              .where((e) =>
                  e.isGrid == true &&
                  (e.propertyType == PropertyTypes.Text ||
                      e.propertyType == PropertyTypes.Boolean ||
                      e.propertyType == PropertyTypes.Number) &&
                  (e.language == null ||
                      e.language == Language.notSelected ||
                      e.language == LocalConfigService.language))
              .toList(),
          data: _selectedData!['records'] ?? [],
        )),
        Expanded(
            child: _selectedDataDetail == null
                ? Container()
                : EntityFormPage(widget.entity.dbName,
                    data: _selectedDataDetail,
                    readOnly: true,
                    loadData: false,
                    isPopup: true,
                    key: Key(DateTime.now().microsecondsSinceEpoch.toString())))
      ],
    );
  }
}

==> tecfy_corev2_package-1.1.30.tar.gz: lib/views/merge/entity_merge.page.dart <==
part of tecfy_corev2_client_package;

class EntityMergePage extends StatefulWidget {
  /// entity for merge
  final Entity entity;

  /// returns true if the user has merged anything
  const EntityMergePage({Key? key, required this.entity}) : super(key: key);

  @override
  _EntityMergePageState createState() => _EntityMergePageState();
}

class _EntityMergePageState extends TecfyState<EntityMergePage>
    with SingleTickerProviderStateMixin {
  String? propertyId;
  List<Map<String, dynamic>>? _data;
  Map<String, dynamic>? _selectedData;
  Map<String, dynamic>? _selectedDataDetail;
  late TabController _tabController;
  Future<void> _onOpenDetailPage(Map<String, dynamic> data) async {
    _selectedData = data;
    if (_selectedData != null) {
      _selectedDataDetail = _selectedData!['records'][0];
      _tabController.animateTo(1);
    }
  }

  Future<void> _onSave() async {
    var data = _data?.where((e) => e['selected'] == true).map((e) {
      return {
        'active': e['default']['_id'],
        'inactive': e['records']
            .where((r) => r['isDefault'] != true)
            .map((r) => r['_id'])
            .toList()
      };
    }).toList();
    if (kDebugMode) print(data);
    if (data == null || data.length == 0) {
      AppNavigator.showMessage(
          context, 'No Data to mirge', MessageType.warning);
      return;
    }
    var result = await TecfyBasicApp.apiService.httpPost(
        'admin/setup/entity/setMerge/${widget.entity.dbName}', data, true);
    if (kDebugMode) print(result);
    if (result.success) {
      AppNavigator.pop(context, true);
    } else {
      AppNavigator.showMessage(context, 'Error', MessageType.error,
          message: result.message);
    }
  }

  Future<void> _onLoadData() async {
    AppNavigator.showLoading(context);
    var result = await TecfyBasicApp.apiService.httpGetDynamic(
        'admin/setup/entity/getMerge/${widget.entity.dbName}?property=$propertyId');
    AppNavigator.pop(context);
    if (result.success && result.data != null) {
      var dd = result.data;
      //print(_data);
      // for (var d in dd) {
      //   d['value'] = ;
      // }
      _data = dd.map<Map<String, dynamic>>((e) {
        return {
          'id': e['key'],
          'selected': false,
          'default': e['value'][0],
          'display': e['value'][0][propertyId],
          'recordsCount': e['value'].length,
          'records': e['value']
              .map<Map<String, dynamic>>((e) => e as Map<String, dynamic>)
              .toList(),
        };
      }).toList();
      setState(() {});
    } else {
      AppNavigator.showMessage(context, 'Error', MessageType.error,
          message: result.message);
    }
  }

  @override
  initState() {
    _tabController = TabController(length: 2, vsync: this);
    _tabController.addListener(() {
      setState(() {});
    });
    super.initState();
  }

  @override
  dispose() {
    _tabController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return BasePage(
      context,
      '${trans('Merge')} ${widget.entity.name}',
      widget.entity.dbName,
      isMobile: LayoutService.isMobile(context),
      appBarActions: [IconButton(icon: Icon(Icons.save), onPressed: _onSave)],
      appBarBottom: LayoutService.isMobile(context)
          ? _getTabBar()
          : Align(
              alignment: LocalConfigService.isArabic
                  ? Alignment.centerRight
                  : Alignment.centerLeft,
              child: Container(
                  child: _getTabBar(),
                  constraints: BoxConstraints(maxWidth: 300))),
      child: Align(
          alignment: Alignment.topCenter,
          child: Container(
              constraints:
                  BoxConstraints(maxWidth: ThemeService.containerWidth),
              child: TabBarView(controller: _tabController, children: [
                _getEntityList(),
                _getEntityMerge(),
              ]))),
    );
  }

  Widget _getEntityList() {
    return Column(children: [
      Row(
        children: [
          Container(
              constraints: BoxConstraints(maxWidth: 250),
              child: DropdownComponenet(
                label: 'Merge Field',
                options: widget.entity.properties
                    .where(
                        (e) => e.propertyType == PropertyTypes.Text && e.isEdit)
                    .map((e) => e.toJson())
                    .toList(),
                idProperty: 'dbName',
                displayProperty: 'nameAr',
                onChanged: (v) => propertyId = v,
              )),
          IconButton(icon: Icon(Icons.refresh), onPressed: _onLoadData),
        ],
      ),
      Expanded(
          child: AppDataTable(
        onRowTap: _onOpenDetailPage,
        isSelectedField: 'selected',
        columns: [
          PropertyText(
              dbName: 'display',
              nameAr: 'البيان',
              nameEn: 'Name',
              readOnly: true),
          PropertyNumber(
              dbName: 'recordsCount',
              nameAr: 'عدد السجلات',
              nameEn: 'Count',
              readOnly: true),
        ],
        data: _data ?? [],
      ))
    ]);
  }

  TabBar _getTabBar() {
    return TabBar(
        controller: _tabController,
        labelColor: ThemeService.mainColor,
        tabs: [
          Tab(child: TecfyText(trans('Merge Data'))),
          Tab(child: TecfyText(trans('Details'))),
        ]);
  }

  Widget _getEntityMerge() {
    if (_selectedData == null)
      return Center(
          child: TecfyText('Please Select '
              'Item'));
    return Row(
      crossAxisAlignment: CrossAxisAlignment.start,
      mainAxisAlignment: MainAxisAlignment.start,
      children: [
        Expanded(
            child: AppDataTable(
          readOnly: true,
          onRowTap: (v) {
            _selectedDataDetail = v;
            setState(() {});
          },
          isSelectedField: 'isDefault',
          onSelectedChanged: (data, selected) {
            if (selected) {
              for (var d in _selectedData!['records']) {
                if (d != data) d['isDefault'] = false;
              }
              _selectedData!['default'] = data;
              _selectedData!['display'] = data[propertyId];
              _selectedData!['selected'] = true;
            } else {
              _selectedData!['default'] = null;
            }
          },
          columns: widget.entity.properties
              .where((e) =>
                  e.isGrid == true &&
                  (e.propertyType == PropertyTypes.Text ||
                      e.propertyType == PropertyTypes.Boolean ||
                      e.propertyType == PropertyTypes.Number) &&
                  (e.language == null ||
                      e.language == Language.notSelected ||
                      e.language == LocalConfigService.language))
              .toList(),
          data: _selectedData!['records'] ?? [],
        )),
        Expanded(
            child: _selectedDataDetail == null
                ? Container()
                : EntityFormPage(widget.entity.dbName,
                    data: _selectedDataDetail,
                    readOnly: true,
                    loadData: false,
                    isPopup: true,
                    key: Key(DateTime.now().microsecondsSinceEpoch.toString())))
      ],
    );
  }
}
